
renoise.tool():add_menu_entry {
  name = "Sample FX Mixer:Set Up Bypass Hydra(s)",
  invoke = function()set_up_bypass_hydras()end  
}

--------------------------------------------------------------------------------
--helper function : custom status message --prefixes tool name and adds brackets 
--------------------------------------------------------------------------------
local function status(message)
  renoise.app():show_status(TOOL_NAME.." Tool: ("..message..")")
end


--function for menu, adds hydras to enable bypassing of all instrument DSPs via a single macro
-------------------------------
function set_up_bypass_hydras()
-------------------------------
 
  local song = renoise.song()
  --selected instrument in renoise
  local cur_inst = song.selected_instrument
  
  --no device chains found
  if #cur_inst.sample_device_chains == 0 then
    status("No Devices In Instrument Present To Bypass")
    return
  end
  
  --default the hydra chain to the first chain
  local hydra_chain = nil
  local hydra = nil  
  local cur_chain = nil
  
  --loop chains to find hydra chain if present
  for chain = 1,#cur_inst.sample_device_chains do
    if cur_inst.sample_device_chains[chain].name == "Hydra Bypass" then
      hydra_chain = cur_inst.sample_device_chains[chain]
      --select chain
      song.selected_sample_device_chain_index = chain
      break       
    end
  end
  
  --sample device chain found but no hydra chain
  if hydra_chain == nil then
    --insert new at 1
    hydra_chain = cur_inst:insert_sample_device_chain_at(1) 
    hydra_chain.name = "Hydra Bypass"
    hydra = hydra_chain:insert_device_at("Audio/Effects/Native/*Hydra", 2)
    hydra.display_name = "BP Hydra"
    song.selected_sample_device_chain_index = 1
  else 
    --loop to delete old hydras (leaving "Bypass Hydra" at pos 2)
    for i = #hydra_chain.devices,3,-1 do
      hydra_chain:delete_device_at(i)
    end
    --if "BP Hydra" Found
    if #hydra_chain.devices > 1 and hydra_chain:device(2).display_name == "BP Hydra" then
      hydra = hydra_chain:device(2)
    else
      --no "BP Hydra" so clear chain completely
      for i = #hydra_chain.devices,2,-1 do
        hydra_chain:delete_device_at(i)
      end
      --else add new one
      hydra = hydra_chain:insert_device_at("Audio/Effects/Native/*Hydra", (#hydra_chain.devices + 1))
      hydra.display_name = "BP Hydra"
    end
  end
  
  --get all devices in the inst fx lanes
  local bypass_params = {}
  local hydra_chain_index = nil
  local counter = 0
  
  --loop fx chains and to get on/off bypass parameters (chain 1 is hydra so no need to scan)
  for fx_chain = 1,#cur_inst.sample_device_chains do
    local cur_chain = cur_inst.sample_device_chains[fx_chain]
    --filter out the hydra chain
    if cur_chain.name ~= "Hydra Bypass" then
      
      --loop devices
      for dev = 2,#cur_chain.devices do
        counter = counter + 1
        bypass_params[counter] = {}
        bypass_params[counter].chain_index = fx_chain-- = cur_chain:device(dev).is_active_parameter
        bypass_params[counter].device_pos = dev
      end
    else
      --get Hydra Bypass index
      hydra_chain_index = fx_chain
    end
  end
  
  --counter for hydras 9 slots
  local hydra_slot = 1
  --now add bypass params to hydra 
  for i = 1,#bypass_params do
    
    for params = 1,#hydra.parameters do 
      if hydra:parameter(params).name == "Out"..hydra_slot.." Track" then
        hydra:parameter(params).value = bypass_params[i].chain_index - 1 --offset due to hydra menu entries being offset
      end
      if hydra:parameter(params).name == "Out"..hydra_slot.." Effect" then
        hydra:parameter(params).value = bypass_params[i].device_pos - 1 --offset due to hydra menu entries being offset
      end
      if hydra:parameter(params).name == "Out"..hydra_slot.." Parameter" then
        hydra:parameter(params).value = 0   --sel_parameter index
      end 
    end
     
    --on slot 8, check if another hydra will be needed.  If so, add it and do necessary 'wiring' from current hydra
    hydra_slot = hydra_slot + 1
    if hydra_slot == 9 and bypass_params[i+2] ~= nil then
      --add a new hydra
      hydra_chain:insert_device_at("Audio/Effects/Native/*Hydra", (#hydra_chain.devices + 1))
      for params = 1,#hydra.parameters do
        --set slot 9 on current hydra to point to the input of the new hydra
        if hydra:parameter(params).name == "Out"..hydra_slot.." Track" then
          hydra:parameter(params).value = (hydra_chain_index - 1) --this chain
        end
        if hydra:parameter(params).name == "Out"..hydra_slot.." Effect" then
          hydra:parameter(params).value =  (#hydra_chain.devices - 1) --offset due to hydra menu entries being offset
        end
        if hydra:parameter(params).name == "Out"..hydra_slot.." Parameter" then
          hydra:parameter(params).value = 1   --set to input of next hydra
        end 
      end 
      --set new hydra as the new target object
      hydra = hydra_chain.devices[#hydra_chain.devices]
      --reset slot counter
      hydra_slot = 1
    end --if
  end--for
  
  status("Bypass Hydras Have Been Set Up")
  
  song.selected_sample_device_index = 2
  song.selected_sample_device.is_maximized = true
  -----------------------------
  --NOTE: Hydra parameter names
  -----------------------------

  --Input
  --Out1 Track  < sets the fx chain (int)
  --Out1 Effect  <sets fx
  --Out1 Parameter  --value is zero for bypass param
  --Out1 Min
  --Out1 Max

end

























